@@ -6,26 +6,14 @@ from distutils.version import Version
 
 import kubernetes
 import urllib3
+import yaml
 from openshift.dynamic import DynamicClient
-from openshift.dynamic.exceptions import (
-    ConflictError,
-    InternalServerError,
-    NotFoundError,
-    ServerTimeoutError,
-)
+from openshift.dynamic.exceptions import InternalServerError, NotFoundError
 from urllib3.exceptions import ProtocolError
 
 from ocp_resources.utils import TimeoutExpiredError, TimeoutSampler
 
 
-DEFAULT_CLUSTER_RETRY_EXCEPTIONS = (
-    ConnectionAbortedError,
-    ConnectionResetError,
-    InternalServerError,
-    ServerTimeoutError,
-    ConflictError,
-)
-
 LOGGER = logging.getLogger(__name__)
 TIMEOUT = 240
 MAX_SUPPORTED_API_VERSION = "v1"
@@ -214,8 +202,6 @@ class ValueMismatch(Exception):
     Raises when value doesn't match the class value
     """
 
-    pass
-
 
 class Resource(object):
     """
@@ -266,7 +252,6 @@ class Resource(object):
         ADMISSIONREGISTRATION_K8S_IO = "admissionregistration.k8s.io"
         APIEXTENSIONS_K8S_IO = "apiextensions.k8s.io"
         APIREGISTRATION_K8S_IO = "apiregistration.k8s.io"
-        APP_KUBERNETES_IO = "app.kubernetes.io"
         APPS = "apps"
         CDI_KUBEVIRT_IO = "cdi.kubevirt.io"
         CONFIG_OPENSHIFT_IO = "config.openshift.io"
@@ -310,16 +295,20 @@ class Resource(object):
         TEMPLATE_OPENSHIFT_IO = "template.openshift.io"
         UPLOAD_CDI_KUBEVIRT_IO = "upload.cdi.kubevirt.io"
         V2V_KUBEVIRT_IO = "v2v.kubevirt.io"
-        VM_KUBEVIRT_IO = "vm.kubevirt.io"
 
     class ApiVersion:
         V1 = "v1"
         V1BETA1 = "v1beta1"
         V1ALPHA1 = "v1alpha1"
-        V1ALPHA3 = "v1alpha3"
 
     def __init__(
-        self, name, client=None, teardown=True, timeout=TIMEOUT, privileged_client=None
+        self,
+        name=None,
+        client=None,
+        teardown=True,
+        timeout=TIMEOUT,
+        privileged_client=None,
+        yaml_file=None,
     ):
         """
         Create a API resource
@@ -335,6 +324,11 @@ class Resource(object):
         self.name = name
         self.client = client
         self.privileged_client = privileged_client
+        self.yaml_file = yaml_file
+        self.resource_dict = None  # Filled in case yaml_file is not None
+        if not (self.name or self.yaml_file):
+            raise ValueError("name or yaml file is required")
+
         if not self.client:
             try:
                 self.client = DynamicClient(
@@ -361,6 +355,18 @@ class Resource(object):
         return sub_resource_level(cls, NamespacedResource, Resource)
 
     def _base_body(self):
+        """
+        Generate resource dict from yaml if self.yaml_file else return base resource dict.
+
+        Returns:
+            dict: Resource dict.
+        """
+        if self.yaml_file:
+            with open(self.yaml_file, "r") as stream:
+                self.resource_dict = yaml.safe_load(stream=stream.read())
+                self.name = self.resource_dict["metadata"]["name"]
+                return self.resource_dict
+
         return {
             "apiVersion": self.api_version,
             "kind": self.kind,
@@ -387,7 +393,6 @@ class Resource(object):
     def clean_up(self):
         if os.environ.get("CNV_TEST_COLLECT_LOGS", "0") == "1":
             try:
-                LOGGER.info(f"Collecting data for {self.kind} {self.name}")
                 _collect_data(resource_object=self)
             except Exception as exception_:
                 LOGGER.warning(exception_)
@@ -620,17 +625,14 @@ class Resource(object):
         self.api().replace(body=resource_dict, name=self.name, namespace=self.namespace)
 
     @staticmethod
-    def retry_cluster_exceptions(
-        func, exceptions_tuple=DEFAULT_CLUSTER_RETRY_EXCEPTIONS, **kwargs
-    ):
-
+    def _retry_etcd_changed(func):
         sampler = TimeoutSampler(
             wait_timeout=10,
             sleep=1,
             func=func,
-            exceptions=exceptions_tuple,
+            exceptions=InternalServerError,
+            exceptions_msg="etcdserver: leader changed",
             print_log=False,
-            **kwargs,
         )
         for sample in sampler:
             return sample
@@ -658,7 +660,7 @@ class Resource(object):
             except TypeError:
                 yield cls(client=dyn_client, name=_resources.metadata.name)
 
-        return Resource.retry_cluster_exceptions(func=_get)
+        return Resource._retry_etcd_changed(func=_get)
 
     @property
     def instance(self):
@@ -672,7 +674,7 @@ class Resource(object):
         def _instance():
             return self.api().get(name=self.name)
 
-        return self.retry_cluster_exceptions(func=_instance)
+        return self._retry_etcd_changed(func=_instance)
 
     @property
     def labels(self):
@@ -761,12 +763,13 @@ class NamespacedResource(Resource):
 
     def __init__(
         self,
-        name,
-        namespace,
+        name=None,
+        namespace=None,
         client=None,
         teardown=True,
         timeout=TIMEOUT,
         privileged_client=None,
+        yaml_file=None,
     ):
         super().__init__(
             name=name,
@@ -774,18 +777,20 @@ class NamespacedResource(Resource):
             teardown=teardown,
             timeout=timeout,
             privileged_client=privileged_client,
+            yaml_file=yaml_file,
         )
         self.namespace = namespace
+        if not (self.name and self.namespace) and not self.yaml_file:
+            raise ValueError("name and namespace or yaml file is required")
 
     @classmethod
-    def get(cls, dyn_client, singular_name=None, raw=False, *args, **kwargs):
+    def get(cls, dyn_client, singular_name=None, *args, **kwargs):
         """
         Get resources
 
         Args:
             dyn_client (DynamicClient): Open connection to remote cluster
             singular_name (str): Resource kind (in lowercase), in use where we have multiple matches for resource
-            raw (bool): If True return raw object from openshift-restclient-python
 
 
         Returns:
@@ -796,23 +801,17 @@ class NamespacedResource(Resource):
         )
         try:
             for resource_field in _resources.items:
-                if raw:
-                    yield resource_field
-                else:
-                    yield cls(
-                        client=dyn_client,
-                        name=resource_field.metadata.name,
-                        namespace=resource_field.metadata.namespace,
-                    )
-        except TypeError:
-            if raw:
-                yield _resources
-            else:
                 yield cls(
                     client=dyn_client,
-                    name=_resources.metadata.name,
-                    namespace=_resources.metadata.namespace,
+                    name=resource_field.metadata.name,
+                    namespace=resource_field.metadata.namespace,
                 )
+        except TypeError:
+            yield cls(
+                client=dyn_client,
+                name=_resources.metadata.name,
+                namespace=_resources.metadata.namespace,
+            )
 
     @property
     def instance(self):
@@ -825,11 +824,19 @@ class NamespacedResource(Resource):
         return self.api().get(name=self.name, namespace=self.namespace)
 
     def _base_body(self):
-        return {
-            "apiVersion": self.api_version,
-            "kind": self.kind,
-            "metadata": {"name": self.name, "namespace": self.namespace},
-        }
+        res = super(NamespacedResource, self)._base_body()
+        if self.yaml_file:
+            self.namespace = self.resource_dict["metadata"].get(
+                "namespace", self.namespace
+            )
+
+        if not self.namespace:
+            raise ValueError("Namespace must be passed or specified in the YAML file.")
+
+        if not self.yaml_file:
+            res["metadata"]["namespace"] = self.namespace
+
+        return res
 
     def to_dict(self):
         return self._base_body()
@@ -930,12 +937,12 @@ class ResourceEditor(object):
         }
 
         # apply changes
-        self._apply_patches_sampler(
+        self._apply_patches(
             patches=patches_to_apply, action_text="Updating", action=self.action
         )
 
     def restore(self):
-        self._apply_patches_sampler(
+        self._apply_patches(
             patches=self._backups, action_text="Restoring", action=self.action
         )
 
@@ -1034,11 +1041,3 @@ class ResourceEditor(object):
                 resource.update_replace(
                     resource_dict=patch
                 )  # replace the resource metadata
-
-    def _apply_patches_sampler(self, patches, action_text, action):
-        return Resource.retry_cluster_exceptions(
-            func=self._apply_patches,
-            patches=patches,
-            action_text=action_text,
-            action=action,
-        )
